import es
from cfglib import AddonCFG, AddonINI
from langlib import Strings
from playerlib import getPlayer, getPlayerList
from usermsg import saytext2
from weaponlib import getWeaponList

info = es.AddonInfo()
info.name = 'LooneyGrenades'
info.basename = info.name.lower()
info.version = '1.0'
info.url = 'http://addons.eventscripts.com/addons/view/LooneyGrenades'
info.author = 'Satoon101'
info.variable = info.basename + '_version'
es.ServerVar(info.variable, info.version,
        info.name + ' Version').set(info.version)
es.ServerVar(info.variable).makepublic()

addonpath = es.getAddonPath(info.basename)

config = AddonCFG(addonpath + '/lg_config.cfg')

ini = AddonINI(addonpath + '/strings.ini')

lg_team_colors = {'#blue': '#ct', '#red': '#t'}

PlayerList = {}


class Player:
    def __init__(self, userid):
        self.userid = userid
        player = getPlayer(self.userid)
        self.lang = player.lang
        self.isbot = player.isbot

    def sendGrenadeMessage(self, group, player, location):
        if self.isbot:
            return
        location = grenades.lang_strings(location, lang=self.lang)
        text = grenades.lang_strings(group, {
            'playername': player.name, 'location': location}, self.lang)
        index = self.getIndex(text, player.index)
        saytext2(self.userid, index, self.reText(text))

    def sendGrenadeSound(self, sound):
        if not self.isbot:
            es.playsound(self.userid, sound, 1)

    @staticmethod
    def getIndex(text, index):
        if '#team' in text and index:
            return index
        for color in lg_team_colors:
            if not color in text:
                continue
            players = getPlayerList(lg_team_colors[color])
            if players:
                return players[0].index
        if '#grey' in text:
            return -1
        return 0

    @staticmethod
    def reText(text):
        for color, hexvalue in (('#default', '\x01'), ('#green', '\x04'),
          ('#darkgreen', '\x05'), ('#team', '\03'), ('#lightgreen', '\x03'),
          ('#red', '\03'), ('#blue', '\x03')):
            text = text.replace(color, hexvalue)
        return text


class LooneyGrenadeManager:
    def __init__(self):
        grenades = [weapon.basename for weapon in
            getWeaponList('#grenade')]
        team_manager = self.getTeamManager()
        teams = [index for index in es.getEntityIndexes(team_manager)
            if not es.getindexprop(index, 'CTeam.m_szTeamname')
            in ('Unassigned', 'Spectator')]
        if team_manager is None or not grenades or not teams:
            raise NotImplementedError, ('LooneyGrenades does not support ' +
                'game "' + es.getGameName() + '"'), es.unload('looneygrenades')
        self.use_chat_messages = config.cvar('lg_use_chat_messages', 1,
            'Enable/Disable sending chat messages when grenade is thrown')
        self.use_sounds = config.cvar('lg_use_sounds', 1,
            'Enable/Disable playing sounds when grenade is thrown')
        self.grenades = {}
        for weapon in grenades:
            self.grenades[weapon] = {}
            for index in teams:
                team = es.getindexprop(index, 'CTeam.m_iTeamnum')
                teamname = es.getindexprop(index, 'CTeam.m_szTeamname')
                group = teamname + ' ' + weapon
                ini.addValueToGroup('sounds', group, 'radio/ct_fireinhole.wav')
                ini.addValueToGroup(group, 'en', '#team$playername#default ' +
                    '@#green$location#default: Fire in the hole!')
                ini.setGroupComments(group, [])
                self.grenades[weapon][team] = group
        self.lang_strings = Strings(ini)
        self.downloads = frozenset(ini['sounds'].values())

    def doDownloads(self):
        for download in self.downloads:
            es.stringtable('downloadables', 'sound/' + download)

    def weaponFire(self, userid, team, weapon):
        if not team in self.grenades[weapon]:
            return
        group = self.grenades[weapon][team]
        players = [playerid.userid for playerid in
            getPlayerList('#human,#alive') if playerid.team == team]
        player = getPlayer(userid)
        if int(self.use_chat_messages):
            location = es.getplayerprop(
                userid, 'CBasePlayer.m_szLastPlaceName')
            if not location in ini:
                ini.addValueToGroup(location, 'en', location)
                ini.setGroupComments(location, [])
                ini.write()
                self.lang_strings = Strings(ini)
            for playerid in players:
                getUser(playerid).sendGrenadeMessage(group, player, location)
        if int(self.use_sounds):
            sound = ini['sounds'][group]
            for playerid in players:
                getUser(playerid).sendGrenadeSound(sound)

    @staticmethod
    def getTeamManager():
        entlist = es.createentitylist()
        for index in entlist:
            for prop in entlist[index]:
                if 'm_szTeamname' in prop:
                    return es.entitygetvalue(index, 'classname')
        return None

grenades = LooneyGrenadeManager()
config.write()
ini.write()


def getUser(userid):
    userid = str(userid)
    if not userid in PlayerList:
        PlayerList[userid] = Player(userid)
    return PlayerList[userid]


def load():
    es.doblock('corelib/noisy_on')
    config.execute()
    grenades.doDownloads()


def es_map_start(ev):
    grenades.doDownloads()
    PlayerList.clear()


def unload():
    es.doblock('corelib/noisy_off')


def player_disconnect(ev):
    userid = ev['userid']
    if userid in PlayerList:
        del PlayerList[userid]


def weapon_fire(ev):
    weapon = ev['weapon']
    if weapon in grenades.grenades:
        grenades.weaponFire(ev['userid'], int(ev['es_userteam']), weapon)
